/***************************************************************************

 *

 * Copyright (C) 2004-2005  SMSC

 *

 * This program is free software; you can redistribute it and/or

 * modify it under the terms of the GNU General Public License

 * as published by the Free Software Foundation; either version 2

 * of the License, or (at your option) any later version.

 *

 * This program is distributed in the hope that it will be useful,

 * but WITHOUT ANY WARRANTY; without even the implied warranty of

 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

 * GNU General Public License for more details.

 *

 * You should have received a copy of the GNU General Public License

 * along with this program; if not, write to the Free Software

 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 *

 ***************************************************************************

 * File: sh3_se01.c

 ***************************************************************************

 * 04/11/2005 Bryan Whitehead, rev 2

 *    updated platform code to support version 1.14 platform changes

 * 04/15/2005 Bryan Whitehead, rev 3

 *    migrated to perforce

 *    merged .c and .h file

 * 08/29/2005 Phong Le, rev 4

 *    Updated to version 1.20

 * 11/07/2005 Phong Le, rev 5

 *    Updated to version 1.21

 * 08/10/2007 Nancy Lin, rev 6

 *    Updated to version 1.53

 ***************************************************************************

 * NOTE: When making changes to platform specific code please remember to 

 *   update the revision number in the PLATFORM_NAME macro. This is a 

 *   platform specific version number and independent of the 

 *   common code version number. The PLATFORM_NAME macro should be found in

 *   your platforms ".h" file.

 */



static const char date_code[] = "081007";



//for a description of these MACROs see readme.txt



#define PLATFORM_CACHE_LINE_BYTES (16UL)



#define PLATFORM_IRQ_POL	(0UL)

#define PLATFORM_IRQ_TYPE	(0UL)



#define PLATFORM_IRQ		(8UL)

#define PLATFORM_RX_DMA	(TRANSFER_PIO)

#define PLATFORM_TX_DMA	(TRANSFER_PIO)

#define PLATFORM_NAME		"SH3 SE01 r6"



//the dma threshold has not been thoroughly tuned but it is

//  slightly better than using zero

#define PLATFORM_DMA_THRESHOLD (200)



typedef struct _PLATFORM_DATA {

	DWORD dwBitWidth;

	DWORD dwIdRev;

	DWORD dwIrq;

	void * dev_id;

} PLATFORM_DATA, *PPLATFORM_DATA;



inline void Platform_SetRegDW(

		DWORD dwLanBase,

		DWORD dwOffset,

		DWORD dwVal)

{

	(*(volatile DWORD *)(dwLanBase+dwOffset))=dwVal;

}



inline DWORD Platform_GetRegDW(

	DWORD dwLanBase,

	DWORD dwOffset)

{

	return (*(volatile DWORD *)(dwLanBase+dwOffset));

}



//See readme.txt for a description of how these

//functions must be implemented

DWORD Platform_Initialize(

	PPLATFORM_DATA platformData,

	DWORD dwLanBase,

	DWORD dwBusWidth);

void Platform_CleanUp(

	PPLATFORM_DATA platformData);

BOOLEAN Platform_Is16BitMode(

	PPLATFORM_DATA platformData);

BOOLEAN Platform_RequestIRQ(

	PPLATFORM_DATA platformData,

	DWORD dwIrq,

	irqreturn_t (*pIsr)(int irq,void *dev_id,struct pt_regs *regs),

	void *dev_id);

DWORD Platform_CurrentIRQ(

	PPLATFORM_DATA platformData);

void Platform_FreeIRQ(

	PPLATFORM_DATA platformData);

BOOLEAN Platform_IsValidDmaChannel(DWORD dwDmaCh);

BOOLEAN Platform_DmaInitialize(

	PPLATFORM_DATA platformData,

	DWORD dwDmaCh);

BOOLEAN Platform_DmaDisable(

	PPLATFORM_DATA platformData,

	const DWORD dwDmaCh);

void Platform_CacheInvalidate(

	PPLATFORM_DATA platformData,

	const void * const pStartAddress,

	const DWORD dwLengthInBytes);

void Platform_CachePurge(

	PPLATFORM_DATA platformData,

	const void * const pStartAddress,

	const DWORD dwLengthInBytes);

DWORD Platform_RequestDmaChannel(

	PPLATFORM_DATA platformData);

void Platform_ReleaseDmaChannel(

	PPLATFORM_DATA platformData,

	DWORD dwDmaChannel);

BOOLEAN Platform_DmaStartXfer(

	PPLATFORM_DATA platformData,

	const DMA_XFER * const pDmaXfer);

DWORD Platform_DmaGetDwCnt(

	PPLATFORM_DATA platformData,

	const DWORD dwDmaCh);

void Platform_DmaComplete(

	PPLATFORM_DATA platformData,

	const DWORD dwDmaCh);

void Platform_GetFlowControlParameters(

	PPLATFORM_DATA platformData,

	PFLOW_CONTROL_PARAMETERS flowControlParameters,

	BOOLEAN useDma);

void Platform_WriteFifo(

	DWORD dwLanBase,

	DWORD *pdwBuf,

	DWORD dwDwordCount);

void Platform_ReadFifo(

	DWORD dwLanBase,

	DWORD *pdwBuf,

	DWORD dwDwordCount);





/************************/

/*         LED          */

/************************/

/* 01.04.16 CommentOut	*/

/* (JG) 1/5/04 Use this port to monitor validation code for 11x. */

#define	LEDREG	(*(volatile unsigned short *)0xB0C00000UL)



/************************/

/*       3.MMU          */

/************************/

#define	PTEH	(*(volatile unsigned long *)0xFFFFFFF0UL)

#define	PTEL	(*(volatile unsigned long *)0xFFFFFFF4UL)

#define	TTB		(*(volatile unsigned long *)0xFFFFFFF8UL)

#define	TEA		(*(volatile unsigned long *)0xFFFFFFFCUL)



#define	MMUCR	(*(volatile unsigned long *)0xFFFFFFE0UL)



/************************/

/*       4.EXP          */

/************************/

#define	TRA		(*(volatile unsigned long *)0xFFFFFFD0UL)

#define	EXPEVT	(*(volatile unsigned long *)0xFFFFFFD4UL)

#define	INTEVT	(*(volatile unsigned long *)0xFFFFFFD8UL)

#define	INTEVT2	(*(volatile unsigned long *)0xA4000000UL)



/**************************/

/*       5.CACHE          */

/**************************/

#define	CCR		(*(volatile unsigned long *)0xFFFFFFECUL)

 #define	CCR_CF_		0x08UL		// Cache Flush Bit



#define	CCR2	(*(volatile unsigned long *)0xA40000B0UL)



/*

 ****************************************************************************

 *	Cache stuff

 ****************************************************************************

 *

 */

//#define CACHE_LINE_BYTES		16UL	//Cache size = 16bytes, for Platform.H

//use PLATFORM_CACHE_LINE_BYTES instead



//#define CACHE_PHYSADDR_MASK		0x1FFFFC00UL

#define CACHE_PHYSADDR_MASK		0xFFFFFC00UL

#define CACHE_OC_ADDRESS_ARRAY	0xF0000000UL

#define CACHE_OC_ENTRY_MASK		0x00000FF0UL

#define CACHE_OC_ASSOC_BIT_		0x00000008UL



/**************************/

/*       6.X/YMemory      */

/**************************/





/**************************/

/*       7.INTC           */

/**************************/

#define	ICR0		(*(volatile unsigned short *)0xFFFFFEE0UL)

#define	ICR1		(*(volatile unsigned short *)0xA4000010UL)

#define	ICR2		(*(volatile unsigned short *)0xA4000012UL)

#define	ICR3		(*(volatile unsigned short *)0xA4000228UL)

#define	PINTER		(*(volatile unsigned short *)0xA4000014UL)

#define	IPRA		(*(volatile unsigned short *)0xFFFFFEE2UL)

#define	IPRB		(*(volatile unsigned short *)0xFFFFFEE4UL)

#define	IPRC		(*(volatile unsigned short *)0xA4000016UL)

#define	IPRD		(*(volatile unsigned short *)0xA4000018UL)

#define	IPRE		(*(volatile unsigned short *)0xA400001AUL)

#define	IPRF		(*(volatile unsigned short *)0xA4000220UL)

#define	IPRG		(*(volatile unsigned short *)0xA4000222UL)

#define	IRR0		(*(volatile unsigned char *)0xA4000004UL)

#define	IRR1		(*(volatile unsigned char *)0xA4000006UL)

#define	IRR2		(*(volatile unsigned char *)0xA4000008UL)

#define	IRR3		(*(volatile unsigned short *)0xA4000224UL)

#define	IRR4		(*(volatile unsigned short *)0xA4000226UL)



/**************************/

/*       8.UBC            */

/**************************/

#define	BARA		(*(volatile unsigned long *)0xFFFFFFB0UL)

#define	BAMRA		(*(volatile unsigned long *)0xFFFFFFB4UL)

#define	BBRA		(*(volatile unsigned short *)0xFFFFFFB8UL)



#define	BARB		(*(volatile unsigned long *)0xFFFFFFA0UL)

#define	BAMRB		(*(volatile unsigned long *)0xFFFFFFA4UL)

#define	BBRB		(*(volatile unsigned short *)0xFFFFFFA8UL)

#define	BDRB		(*(volatile unsigned long *)0xFFFFFF90UL)

#define	BDMRB		(*(volatile unsigned long *)0xFFFFFF94UL)

#define	BRCR		(*(volatile unsigned long *)0xFFFFFF98UL)



#define	BETR		(*(volatile unsigned short *)0xFFFFFF9CUL)

#define	BRSR		(*(volatile unsigned long *)0xFFFFFFACUL)

#define	BRDR		(*(volatile unsigned long *)0xFFFFFFBCUL)



#define	BASRA		(*(volatile unsigned char *)0xFFFFFFE4UL)

#define	BASRB		(*(volatile unsigned char *)0xFFFFFFE8UL)



/**************************/

/*       9.Low Power      */

/**************************/

#define	STBCR		(*(volatile unsigned char *)0xFFFFFF82UL)

#define	STBCR2		(*(volatile unsigned char *)0xFFFFFF88UL)

#define	STBCR3		(*(volatile unsigned char *)0xA4000230UL)

#define	SRSTR		(*(volatile unsigned char *)0xA4000232UL)



/**************************/

/*      10.CPG            */

/**************************/

#define	FRQCR		(*(volatile unsigned short *)0xFFFFFF80UL)



#define	WTCNT_R		(*(volatile unsigned char *)0xFFFFFF84UL)

#define	WTCNT_W		(*(volatile unsigned short *)0xFFFFFF84UL)

#define	WTCSR_R		(*(volatile unsigned char *)0xFFFFFF86UL)

#define	WTCSR_W		(*(volatile unsigned short *)0xFFFFFF86UL)



/**************************/

/*      11.EXCPG          */

/**************************/

#define	EXCPGCR		(*(volatile unsigned char *)0xA4000236UL)



/**************************/

/*      12.BSC            */

/**************************/

#define	BCR1		(*(volatile unsigned short *)0xFFFFFF60UL)

#define	BCR2		(*(volatile unsigned short *)0xFFFFFF62UL)

#define	WCR1		(*(volatile unsigned short *)0xFFFFFF64UL)

#define	WCR2		(*(volatile unsigned short *)0xFFFFFF66UL)

 /* Bus Wait State Control Definitions */

 #define	WAIT_STATES_0		0x0

 #define	WAIT_STATES_1		0x1

 #define	WAIT_STATES_2		0x2

 #define	WAIT_STATES_3		0x3

 #define	WAIT_STATES_4		0x4

 #define	WAIT_STATES_6		0x5

 #define	WAIT_STATES_8		0x6

 #define	WAIT_STATES_10		0x7



 /* Bit map in WCR2 */

 #define	AREA_0				0

 #define	AREA_2				2

 #define	AREA_3				3

 #define	AREA_4				4

 #define	AREA_5				5

 #define	AREA_6				6



 #define	AREA0_WAIT_CTRL_	0x0007

 #define	AREA2_WAIT_CTRL_	0x0018

 #define	AREA3_WAIT_CTRL_	0x0060

 #define	AREA4_WAIT_CTRL_	0x0380

 #define	AREA5_WAIT_CTRL_	0x1C00

 #define	AREA6_WAIT_CTRL_	0xE000



#define	MCR			(*(volatile unsigned short *)0xFFFFFF68UL)

#define	PCR			(*(volatile unsigned short *)0xFFFFFF6CUL)

#define	RTCSR		(*(volatile unsigned short *)0xFFFFFF6EUL)

#define	RTCNT		(*(volatile unsigned short *)0xFFFFFF70UL)

#define	RTCOR		(*(volatile unsigned short *)0xFFFFFF72UL)

#define	RFCR		(*(volatile unsigned short *)0xFFFFFF74UL)



#define	SDMR		(*(volatile unsigned char *)0xFFFFE880UL)



/**************************/

/*      13.LBSC           */

/**************************/

/*

	Refer BCR1, BCR2, WCR1, WCR2, MCR

*/



/**************************/

/*      14.DMAC           */

/**************************/

/*

	Chanel 0

*/

#define	SAR0		(*(volatile unsigned long *)0xA4000020UL)

#define	DAR0		(*(volatile unsigned long *)0xA4000024UL)

#define	DMATCR0		(*(volatile unsigned long *)0xA4000028UL)

#define	CHCR0		(*(volatile unsigned long *)0xA400002CUL)



/*

	Chanel 1

*/

#define	SAR1		(*(volatile unsigned long *)0xA4000030UL)

#define	DAR1		(*(volatile unsigned long *)0xA4000034UL)

#define	DMATCR1		(*(volatile unsigned long *)0xA4000038UL)

#define	CHCR1		(*(volatile unsigned long *)0xA400003CUL)



/*

	Chanel 2

*/

#define	SAR2		(*(volatile unsigned long *)0xA4000040UL)

#define	DAR2		(*(volatile unsigned long *)0xA4000044UL)

#define	DMATCR2		(*(volatile unsigned long *)0xA4000048UL)

#define	CHCR2		(*(volatile unsigned long *)0xA400004CUL)



/*

	Chanel 3

*/

#define	SAR3		(*(volatile unsigned long *)0xA4000050UL)

#define	DAR3		(*(volatile unsigned long *)0xA4000054UL)

#define	DMATCR3		(*(volatile unsigned long *)0xA4000058UL)

#define	CHCR3		(*(volatile unsigned long *)0xA400005CUL)



/* Bit map of CHCRx */

 #define	DMAC_ENBL_		0x00000001UL

 #define	XFER_END_		0x00000002UL

 #define	INT_ENBL_		0x00000004UL



 #define	XMIT_SIZE_		0x00000018UL

  #define	XMIT_BYTE		0x00000000UL

  #define	XMIT_WORD		0x00000008UL

  #define	XMIT_DWORD		0x00000010UL

  #define	XMIT_QDWORD		0x00000018UL



 #define	XMIT_MODE_		0x00000020UL



 #define	XMIT_RESRC_		0x00000F00UL

  #define	AUTO_REQ		0x00000400UL



 #define	SRC_ADDR_MODE_	0x00003000UL

  #define	FIXED_SRC_ADDR	0x00000000UL

  #define	INC_SRC_ADDR	0x00001000UL

  #define	DEC_SRC_ADDR	0x00002000UL



 #define	DST_ADDR_MODE_	0x0000C000UL

  #define	FIXED_DST_ADDR	0x00000000UL

  #define	INC_DST_ADDR	0x00004000UL

  #define	DEC_DST_ADDR	0x00008000UL



/*

	Common

*/

#define	DMAOR		(*(volatile unsigned short *)0xA4000060UL)

 #define	DMA_MSTR_EN_	0x0001

 #define	NMI_FLAG_		0x0002

 #define	ADDR_ERR_FLAG_	0x0004



#define	CHRAR		(*(volatile unsigned short *)0xA400022AUL)



/**************************/

/*      15.TMU            */

/**************************/

/*

	Common

*/

#define	TSTR		(*(volatile unsigned char *)0xFFFFFE92UL)

 #define STR2_		0x04

 #define STR1_		0x02

 #define STR0_		0x01



/*

	Chanel 0

*/

#define	TCOR0		(*(volatile unsigned long *)0xFFFFFE94UL)

#define	TCNT0		(*(volatile unsigned long *)0xFFFFFE98UL)

#define	TCR0		(*(volatile unsigned short *)0xFFFFFE9CUL)

 #define PCLK_DIV4_		0x0000

 #define PCLK_DIV16_	0x0001

 #define PCLK_DIV64_	0x0002

 #define PCLK_DIV256_	0x0003

 #define RTCCLK_		0x0004



/*

	Chanel 1

*/

#define	TCOR1		(*(volatile lunsigned ong *)0xFFFFFEA0UL)

#define	TCNT1		(*(volatile unsigned long *)0xFFFFFEA4UL)

#define	TCR1		(*(volatile unsigned short *)0xFFFFFEA8UL)



/*

	Chanel 2

*/

#define	TCOR2		(*(volatile unsigned long *)0xFFFFFEACUL)

#define	TCNT2		(*(volatile unsigned long *)0xFFFFFEB0UL)

#define	TCR2		(*(volatile unsigned short *)0xFFFFFEB4UL)



/**************************/

/*      16.RTC            */

/**************************/

#define	R64CNT		(*(volatile unsigned char *)0xFFFFFEC0UL)

#define	RSECCNT		(*(volatile unsigned char *)0xFFFFFEC2UL)

#define	RMINCNT		(*(volatile unsigned char *)0xFFFFFEC4UL)

#define	RHRCNT		(*(volatile unsigned char *)0xFFFFFEC6UL)

#define	RWKCNT		(*(volatile unsigned char *)0xFFFFFEC8UL)



#define	RDAYCNT		(*(volatile unsigned char *)0xFFFFFECAUL)

#define	RMONCNT		(*(volatile unsigned char *)0xFFFFFECCUL)



#define	RYRCNT		(*(volatile unsigned char *)0xFFFFFECEUL)

#define	RSECAR		(*(volatile unsigned char *)0xFFFFFED0UL)

#define	RMINAR		(*(volatile unsigned char *)0xFFFFFED2UL)

#define	RHRAR		(*(volatile unsigned char *)0xFFFFFED4UL)



#define	RWKAR		(*(volatile unsigned char *)0xFFFFFED6UL)

#define	RDAYAR		(*(volatile unsigned char *)0xFFFFFED8UL)

#define	RMONAR		(*(volatile unsigned char *)0xFFFFFEDAUL)

#define	RCR1		(*(volatile unsigned char *)0xFFFFFEDCUL)

#define	RCR2		(*(volatile unsigned char *)0xFFFFFEDEUL)



/**************************/

/*      17.SCI            */

/**************************/

#define	SCSMR	(*(volatile unsigned char *)0xFFFFFE80UL)

#define	SCBRR	(*(volatile unsigned char *)0xFFFFFE82UL)

#define	SCSCR	(*(volatile unsigned char *)0xFFFFFE84UL)

#define	SCTDR	(*(volatile unsigned char *)0xFFFFFE86UL)

#define SCSSR	(*(volatile unsigned char *)0xFFFFFE88UL)

#define	SCRDR	(*(volatile unsigned char *)0xFFFFFE8AUL)



#define	SCPDR	(*(volatile unsigned char *)0xA4000136UL)



/**************************/

/*      18.SmartC         */

/**************************/

#define	SMART_SCSMR		(*(volatile unsigned char *)0xFFFFFE80UL)

#define	SMART_SCBRR		(*(volatile unsigned char *)0xFFFFFE82UL)

#define	SMART_SCSCR		(*(volatile unsigned char *)0xFFFFFE84UL)

#define	SMART_SCTDR		(*(volatile unsigned char *)0xFFFFFE86UL)

#define SMART_SCSSR		(*(volatile unsigned char *)0xFFFFFE88UL)

#define	SMART_SCRDR		(*(volatile unsigned char *)0xFFFFFE8AUL)

#define	SMART_SCSCMR	(*(volatile unsigned char *)0xFFFFFE8CUL)



/**************************/

/*      19.SCIF2          */

/**************************/

#define	SCSMR2		(*(volatile unsigned char *)0xA4000150UL)

#define	SCBRR2		(*(volatile unsigned char *)0xA4000152UL)

#define	SCSCR2		(*(volatile unsigned char *)0xA4000154UL)

#define	SCSFDR2		(*(volatile unsigned char *)0xA4000156UL)

#define	SCSSR2		(*(volatile unsigned short *)0xA4000158UL)

#define	SCFRDR2		(*(volatile unsigned char *)0xA400015AUL)

#define	SCFCR2		(*(volatile unsigned char *)0xA400015CUL)

#define	SCFDR2		(*(volatile unsigned short *)0xA400015EUL)



#define	SCIRDR2		(*(volatile unsigned short *)0xA400023AUL)



/**************************/

/*      20.SIOF           */

/**************************/

#define	SIMDR		(*(volatile unsigned short *)0xA40000C0UL)

#define	SISCR		(*(volatile unsigned short *)0xA40000C2UL)

#define	SITDAR		(*(volatile unsigned short *)0xA40000C4UL)

#define	SIRDAR		(*(volatile unsigned short *)0xA40000C6UL)

#define	SICDAR		(*(volatile unsigned short *)0xA40000C8UL)

#define	SICTR		(*(volatile unsigned short *)0xA40000CCUL)

#define	SIFCTR		(*(volatile unsigned short *)0xA40000D0UL)

#define	SISTR		(*(volatile unsigned short *)0xA40000D4UL)

#define	SIIER		(*(volatile unsigned short *)0xA40000D6UL)

#define	SITDR		(*(volatile unsigned long *)0xA40000E0UL)

#define	SIRDR		(*(volatile unsigned long *)0xA40000E4UL)

#define	SITCR		(*(volatile unsigned long *)0xA40000E8UL)

#define	SIRCR		(*(volatile unsigned long *)0xA40000ECUL)



/**************************/

/*      21.AFEIF          */

/**************************/

#define	ACTR1	(*(volatile unsigned short *)0xA4000180UL)

#define	ACTR2	(*(volatile unsigned short *)0xA4000182UL)

#define	ASTR1	(*(volatile unsigned short *)0xA4000184UL)

#define	ASTR2	(*(volatile unsigned short *)0xA4000186UL)

#define	MRCR	(*(volatile unsigned short *)0xA4000188UL)

#define	MPCR	(*(volatile unsigned short *)0xA400018AUL)

#define	DPNQ	(*(volatile unsigned short *)0xA400018CUL)

#define	RCNT	(*(volatile unsigned short *)0xA400018EUL)

#define	ACDR	(*(volatile unsigned short *)0xA4000190UL)

#define	ASDR	(*(volatile unsigned short *)0xA4000192UL)

#define	TDFP	(*(volatile unsigned long *)0xA4000194UL)

#define	RDFP	(*(volatile unsigned long *)0xA4000198UL)



/**************************/

/*      22.USBPMPXC       */

/**************************/

#define	EXPFC	(*(volatile unsigned char *)0xA4000234UL)



/**************************/

/*      23.USBF           */

/**************************/

#define	USB_FUNC_IFR0	(*(volatile unsigned char *)0xA4000240UL)



#define	USB_FUNC_IFR1	(*(volatile unsigned char *)0xA4000241UL)

#define	USB_FUNC_EPDR0I	(*(volatile unsigned char *)0xA4000242UL)

#define	USB_FUNC_EPDR0O	(*(volatile unsigned char *)0xA4000243UL)

#define	USB_FUNC_TRG	(*(volatile unsigned char *)0xA4000244UL)

#define	USB_FUNC_FCLR	(*(volatile unsigned char *)0xA4000245UL)



#define	USB_FUNC_EPSZ0O	(*(volatile unsigned char *)0xA4000246UL)

#define	USB_FUNC_EPDR0S	(*(volatile unsigned char *)0xA4000247UL)

#define	USB_FUNC_DASTS	(*(volatile unsigned char *)0xA4000248UL)



#define	USB_FUNC_EPDR2	(*(volatile unsigned char *)0xA4000249UL)

#define	USB_FUNC_ISR0	(*(volatile unsigned char *)0xA400024AUL)

#define	USB_FUNC_EPSTL	(*(volatile unsigned char *)0xA400024BUL)



#define	USB_FUNC_IER0	(*(volatile unsigned char *)0xA400024CUL)

#define	USB_FUNC_IER1	(*(volatile unsigned char *)0xA400024DUL)



#define	USB_FUNC_EPDR1	(*(volatile unsigned char *)0xA400024EUL)

#define	USB_FUNC_EPSZ1	(*(volatile unsigned char *)0xA400024FUL)

#define	USB_FUNC_ISR1	(*(volatile unsigned char *)0xA4000250UL)





#define	USB_FUNC_DMAR	(*(volatile unsigned char *)0xA4000251UL)

#define	USB_FUNC_EPDR3	(*(volatile unsigned char *)0xA4000252UL)



/**************************/

/*      24.USBH           */

/**************************/

#define	USB_HOST_REVR		(*(volatile unsigned long *)0xA4000280UL)

#define	USB_HOST_CTR		(*(volatile unsigned long *)0xA4000284UL)



#define	USB_HOST_CMDSR		(*(volatile unsigned long *)0xA4000288UL)



#define	USB_HOST_IRPSR		(*(volatile unsigned long *)0xA400028CUL)



#define	USB_HOST_IRPER		(*(volatile unsigned long *)0xA4000290UL)

#define	USB_HOST_IRPDR		(*(volatile unsigned long *)0xA4000294UL)

#define	USB_HOST_HCCAR		(*(volatile unsigned long *)0xA4000298UL)

#define	USB_HOST_PCUREDR	(*(volatile unsigned long *)0xA400029CUL)

#define	USB_HOST_CTRHDEDR	(*(volatile unsigned long *)0xA40002A0UL)



#define	USB_HOST_CTRCUREDR	(*(volatile unsigned long *)0xA40002A4UL)



#define	USB_HOST_BULKHDED	(*(volatile unsigned long *)0xA40002A8UL)



#define	USB_HOST_BULKCUREDR	(*(volatile unsigned long *)0xA40002ACUL)

#define	USB_HOST_DONRHDEDR	(*(volatile unsigned long *)0xA40002B0UL)

#define	USB_HOST_FMTNERVR	(*(volatile unsigned long *)0xA40002B4UL)

#define	USB_HOST_FRMREMINGR	(*(volatile unsigned long *)0xA40002B8UL)

#define	USB_HOST_FMNUMBR	(*(volatile unsigned long *)0xA40002BCUL)



#define	USB_HOST_PDSTRR		(*(volatile unsigned long *)0xA40002C0UL)



#define	USB_HOST_LSTSHDR	(*(volatile unsigned long *)0xA40002C4UL)



#define	USB_HOST_RHDESCAR	(*(volatile unsigned long *)0xA40002C8UL)

#define	USB_HOST_RHDESCBR	(*(volatile unsigned long *)0xA40002CCUL)

#define	USB_HOST_RHSR		(*(volatile unsigned long *)0xA40002D0UL)

#define	USB_HOST_RHPTSR1	(*(volatile unsigned long *)0xA40002D4UL)

#define	USB_HOST_RHPTSR2	(*(volatile unsigned long *)0xA40002D8UL)



/**************************/

/*      25.LCD            */

/**************************/

#define	LDICKR		(*(volatile unsigned short *)0xA4000C00UL)

#define	LDMTR		(*(volatile unsigned short *)0xA4000C02UL)

#define	LDDFR		(*(volatile unsigned short *)0xA4000C04UL)

#define	LDSMR		(*(volatile unsigned short *)0xA4000C06UL)

#define	LDSARU		(*(volatile unsigned long *)0xA4000C08UL)

#define	LDSARL		(*(volatile unsigned long *)0xA4000C0CUL)

#define	LDLAOR		(*(volatile unsigned short *)0xA4000C10UL)

#define	LDPALCR		(*(volatile unsigned short *)0xA4000C12UL)



#define	LDPR00		(*(volatile unsigned long *)0xA4000800UL)



#define	LDHCNR		(*(volatile unsigned short *)0xA4000C14UL)

#define	LDHSYNR		(*(volatile unsigned short *)0xA4000C16UL)



#define	LDVDLNR		(*(volatile unsigned short *)0xA4000C18UL)

#define	LDVTLNR		(*(volatile unsigned short *)0xA4000C1AUL)

#define	LDVSYNR		(*(volatile unsigned short *)0xA4000C1CUL)

#define	LDACLNR		(*(volatile unsigned short *)0xA4000C1EUL)

#define	LDINTR		(*(volatile unsigned short *)0xA4000C20UL)

#define	LDPMMR		(*(volatile unsigned short *)0xA4000C24UL)

#define	LDPSPR		(*(volatile unsigned short *)0xA4000C26UL)

#define	LDCNTR		(*(volatile unsigned short *)0xA4000C28UL)



/**************************/

/*      26.IOMPX          */

/**************************/

#define	PACR		(*(volatile unsigned short *)0xA4000100UL)

#define	PBCR		(*(volatile unsigned short *)0xA4000102UL)

#define	PCCR		(*(volatile unsigned short *)0xA4000104UL)

#define	PDCR		(*(volatile unsigned short *)0xA4000106UL)

#define	PECR		(*(volatile unsigned short *)0xA4000108UL)

#define	PFCR		(*(volatile unsigned short *)0xA400010AUL)

#define	PGCR		(*(volatile unsigned short *)0xA400010CUL)

#define	PHCR		(*(volatile unsigned short *)0xA400010EUL)

#define	PJCR		(*(volatile unsigned short *)0xA4000110UL)

#define	PKCR		(*(volatile unsigned short *)0xA4000112UL)

#define	PLCR		(*(volatile unsigned short *)0xA4000114UL)

#define SCPCR		(*(volatile unsigned short *)0x04000116UL)

#define	PMCR		(*(volatile unsigned short *)0xA4000118UL)



/**************************/

/*      27.I/O PORT       */

/**************************/

#define	PADR		(*(volatile unsigned char *)0xA4000120UL)

#define	PBDR		(*(volatile unsigned char *)0xA4000122UL)

#define	PCDR		(*(volatile unsigned char *)0xA4000124UL)

#define	PDDR		(*(volatile unsigned char *)0xA4000126UL)

#define	PEDR		(*(volatile unsigned char *)0xA4000128UL)

#define	PFDR		(*(volatile unsigned char *)0xA400012AUL)

#define	PGDR		(*(volatile unsigned char *)0xA400012CUL)

#define	PHDR		(*(volatile unsigned char *)0xA400012EUL)

#define	PJDR		(*(volatile unsigned char *)0xA4000130UL)

#define	PKDR		(*(volatile unsigned char *)0xA4000132UL)

#define	PLDR		(*(volatile unsigned char *)0xA4000134UL)

//#define	SCPDR		(*(volatile unsigned char *)0xA4000136UL)

#define	PMDR		(*(volatile unsigned char *)0xA4000138UL)



/**************************/

/*      28.A/D            */

/**************************/

#define	ADDRAH		(*(volatile unsigned char *)0xA4000080UL)

#define	ADDRAL		(*(volatile unsigned char *)0xA4000082UL)

#define	ADDRBH		(*(volatile unsigned char *)0xA4000084UL)

#define	ADDRBL		(*(volatile unsigned char *)0xA4000086UL)

#define	ADDRCH		(*(volatile unsigned char *)0xA4000088UL)

#define	ADDRCL		(*(volatile unsigned char *)0xA400008AUL)

#define	ADDRDH		(*(volatile unsigned char *)0xA400008CUL)

#define	ADDRDL		(*(volatile unsigned char *)0xA400008EUL)

#define	ADCSR		(*(volatile unsigned char *)0xA4000090UL)

#define	ADCR		(*(volatile unsigned char *)0xA4000092UL)



/**************************/

/*      29.D/A            */

/**************************/

#define	DADR0		(*(volatile unsigned char *)0xA40000A0UL)

#define	DADR1		(*(volatile unsigned char *)0xA40000A2UL)

#define	DACR		(*(volatile unsigned char *)0xA40000A4UL)



/**************************/

/*      30.PCC            */

/**************************/

#define	PCC0ISR		(*(volatile unsigned char *)0xA40000E0UL)

#define	PCC0GCR		(*(volatile unsigned char *)0xA40000E2UL)

#define	PCC0CSCR	(*(volatile unsigned char *)0xA40000E4UL)

#define	PCC0CSCIER	(*(volatile unsigned char *)0xA40000E6UL)



/**************************/

/*      31.HUDI           */

/**************************/

#define	SDBPR		()

#define	SDIR		(*(volatile unsigned short *)0xA4000200UL)

#define	SDBSR		()



#define LAN_CS	(5UL)

#define LAN_CSBASE (0xB4000000UL)

#define LAN_IRQ PLATFORM_IRQ



#define CpuToPhysicalAddr(cpuAddr) ((DWORD)cpuAddr)



/************************************************************************/

/*	Set Bus wait states for specific memory area			    		*/

/************************************************************************/

static const WORD wWaitArea[7] = {AREA0_WAIT_CTRL_, 0, AREA2_WAIT_CTRL_,

	 	AREA3_WAIT_CTRL_, AREA4_WAIT_CTRL_, AREA5_WAIT_CTRL_, AREA6_WAIT_CTRL_};



static const WORD wWaitState[11] = {WAIT_STATES_0, WAIT_STATES_1, WAIT_STATES_2,

	 	WAIT_STATES_3, WAIT_STATES_4, 0, WAIT_STATES_6, 0,

		WAIT_STATES_8, 0, WAIT_STATES_10};



DWORD Platform_Initialize(

	PPLATFORM_DATA platformData,

	DWORD dwLanBase,DWORD dwBusWidth)

{

	int iAreaNum=0;

	int iWaitStNum=10UL;//initially set to slowest setting incase the chip is 115/116



	WORD wWaitCtrlReg, wTemp;

	WORD wBcrMask=1;

	SMSC_TRACE("--> Platform_Initialize()");

	SMSC_ASSERT(platformData!=NULL);

	platformData->dwBitWidth=0;

	platformData->dwIdRev=0;

	if(dwLanBase==0x0UL) {

		dwLanBase=LAN_CSBASE;

	}



	if((0xA8000000UL<=dwLanBase)&&(dwLanBase<=(0xAC000000UL-LAN_REGISTER_EXTENT))) 

	{

		iAreaNum=2;

	}

	if((0xAC000000UL<=dwLanBase)&&(dwLanBase<=(0xB0000000UL-LAN_REGISTER_EXTENT)))

	{

		iAreaNum=3;

	}

	if((0xB0000000UL<=dwLanBase)&&(dwLanBase<=(0xB4000000UL-LAN_REGISTER_EXTENT)))

	{

		iAreaNum=4;

	}

	if((0xB4000000UL<=dwLanBase)&&(dwLanBase<=(0xB8000000UL-LAN_REGISTER_EXTENT)))

	{

		iAreaNum=5;

	}

	if((0xB8000000UL<=dwLanBase)&&(dwLanBase<=(0xBC000000UL-LAN_REGISTER_EXTENT)))

	{

		iAreaNum=6;

	}

	if(iAreaNum==0) {

		SMSC_WARNING("Invalid Lan Base Address = 0x%08lX",dwLanBase);

		SMSC_WARNING("  must be in Area 2,3,4,5, or 6 and uncached");



		return 0;

	}

	SMSC_TRACE(" Lan Base at 0x%08lX in Area %d",dwLanBase,iAreaNum);

//	SMSC_TRACE("BCR1 = 0x%04X, will not change it",BCR1);



	// Init BSC registers for 11x

//	SMSC_TRACE(" Before: WCR2 = 0x%04X", WCR2);

	wWaitCtrlReg = WCR2;

	wWaitCtrlReg &= (WORD)(~(wWaitArea[iAreaNum]));	// Clear specified Area

	wTemp = wWaitState[iWaitStNum];



	switch (iAreaNum)

	{

		case AREA_2:

			wTemp <<= 3;

			wBcrMask <<=4;

			break;

	

		case AREA_3:

			wTemp <<= 5;

			wBcrMask <<=6;

			break;

	

		case AREA_4:

			wTemp <<= 7;

			wBcrMask <<=8;

			break;

	

		case AREA_5:

			wTemp <<= 10;

			wBcrMask <<=10;

			break;

	

		case AREA_6:

			wTemp <<= 13;

			wBcrMask <<= 12;

			break;

	

		default:

			break;

	}

	

	wWaitCtrlReg |= wTemp;		// Inserted 2 wait states for Area 5 and 2.

	WCR2 = wWaitCtrlReg;

//	SMSC_TRACE(" After:  WCR2 = 0x%04X", WCR2);



	wTemp=BCR2;

//	SMSC_TRACE(" Before: BCR2 = 0x%04X", wTemp);

	if((dwBusWidth!=16)&&(dwBusWidth!=32)) {

		DWORD test=0;

		WORD dummy=0;

		BCR2|=wBcrMask;

#define BYTE_TEST_OFFSET	(0x64)

		test=(*(volatile unsigned long *)(dwLanBase+BYTE_TEST_OFFSET));

		dummy=(*(volatile unsigned short *)(dwLanBase+BYTE_TEST_OFFSET+2));

		if(test==0x87654321UL) {

			SMSC_TRACE(" 32 bit mode detected");

			platformData->dwBitWidth=32;

		} else {

			BCR2&=(~(wBcrMask));

			test=(*(volatile unsigned long *)(dwLanBase+BYTE_TEST_OFFSET));

			if(test==0x87654321UL) {

				SMSC_TRACE(" 16 bit mode detected");

				platformData->dwBitWidth=16;

			} else {

				SMSC_WARNING(" neither 16 nor 32 bit mode detected, test=0x%08lX",test);

				BCR2=wTemp;

				dwLanBase=0;

				dummy=dummy;//make lint happy

			}

		}

	} else if(dwBusWidth==16) {



		DWORD test=0;

		BCR2&=(~(wBcrMask));

		test=(*(volatile unsigned long *)(dwLanBase+BYTE_TEST_OFFSET));

		if(test==0x87654321UL) {

			SMSC_TRACE(" 16 bit mode assigned and verified");

			platformData->dwBitWidth=16;

		} else {

			SMSC_WARNING(" 16 bit mode not verified");

			dwLanBase=0;

		}

	} else if(dwBusWidth==32) {

		DWORD test=0;

		BCR2|=wBcrMask;

		test=(*(volatile unsigned long *)(dwLanBase+BYTE_TEST_OFFSET));

		if(test==0x87654321UL) {

			SMSC_TRACE(" 32 bit mode assigned and verified");

			platformData->dwBitWidth=32;

		} else {

			SMSC_WARNING(" 32 bit mode not verified");

			dwLanBase=0;

		}

	} else {

		SMSC_ASSERT(FALSE);

	}

//	SMSC_TRACE(" After: BCR2 = 0x%04X", BCR2);



	if(dwLanBase!=0)

	{

		DWORD idRev=0;

#define ID_REV_OFFSET	(0x50)

		idRev=(*(volatile unsigned long *)(dwLanBase+ID_REV_OFFSET));

		platformData->dwIdRev=idRev;

		switch(idRev&0xFFFF0000UL) {

		case 0x01180000UL:

		case 0x01170000UL:

		case 0x01120000UL:

		case 0x92110000UL:

		case 0x92210000UL:

			iWaitStNum=4UL;

			wWaitCtrlReg = WCR2;

			wWaitCtrlReg &= (WORD)(~(wWaitArea[iAreaNum]));	// Clear specified Area

			wTemp = wWaitState[iWaitStNum];



			switch (iAreaNum)

			{

				case AREA_2:

					wTemp <<= 3;

					wBcrMask <<=4;

					break;

	

				case AREA_3:

					wTemp <<= 5;

					wBcrMask <<=6;

					break;

		

				case AREA_4:

					wTemp <<= 7;

					wBcrMask <<=8;

					break;

	

				case AREA_5:

					wTemp <<= 10;

					wBcrMask <<=10;

					break;

	

				case AREA_6:

					wTemp <<= 13;

					wBcrMask <<= 12;

					break;

	

				default:break;//make lint happy

			}

	

			wWaitCtrlReg |= wTemp;		// Inserted 2 wait states for Area 5 and 2.

			WCR2 = wWaitCtrlReg;

			break;	

		default:break;//make lint happy

		}

	}



	SMSC_TRACE("<-- Platform_Initialize()");

	return dwLanBase;

}



void Platform_CleanUp(

	PPLATFORM_DATA platformData)

{

}



BOOLEAN Platform_Is16BitMode(

	PPLATFORM_DATA platformData)

{

	SMSC_ASSERT(platformData!=NULL);

	if(platformData->dwBitWidth==16) {

		return TRUE;

	}

	return FALSE;

}



void Platform_GetFlowControlParameters(

	PPLATFORM_DATA platformData,

	PFLOW_CONTROL_PARAMETERS flowControlParameters,

	BOOLEAN useDma)

{

	memset(flowControlParameters,0,sizeof(FLOW_CONTROL_PARAMETERS));

	flowControlParameters->BurstPeriod=100;

	flowControlParameters->IntDeas=0;

	if(useDma) {

		if(Platform_Is16BitMode(platformData)) {

			switch(platformData->dwIdRev&0xFFFF0000) {

			case 0x01180000UL:

			case 0x01170000UL:

			case 0x01120000UL:

			case 0x92110000UL: // Boylston Lite, not tested

			case 0x92210000UL: // Bedford, not tested

				//117/118,16 bit,DMA

				flowControlParameters->MaxThroughput=(0xABE0AUL);

				flowControlParameters->MaxPacketCount=(0x1D1UL);

				flowControlParameters->PacketCost=(0x1CEUL);

				flowControlParameters->BurstPeriod=(0x51UL);

				flowControlParameters->IntDeas=(0x09UL);

				break;

			case 0x01160000UL:

			case 0x01150000UL:

			case 0x92100000UL: // Boylston Lite, not tested

			case 0x92200000UL: // Bedford, not tested

				//115/116,16 bit,DMA

				flowControlParameters->MaxThroughput=0x89DF2UL;

				flowControlParameters->MaxPacketCount=0x175UL;

				flowControlParameters->PacketCost=0x167UL;

				flowControlParameters->BurstPeriod=0x3EUL;

				flowControlParameters->IntDeas=0x05UL;

				break;

			default:break;//make lint happy

			}

		} else {

			switch(platformData->dwIdRev&0xFFFF0000) {

			case 0x01180000UL:

			case 0x01170000UL:

			case 0x01120000UL:

				//117/118,32 bit,DMA

				flowControlParameters->MaxThroughput=(0xC7F82UL);

				flowControlParameters->MaxPacketCount=(0x21DUL);

				flowControlParameters->PacketCost=(0x17UL);

				flowControlParameters->BurstPeriod=(0x1EUL);

				flowControlParameters->IntDeas=(0x17UL);

				break;

			case 0x01160000UL:

			case 0x01150000UL:

				//115/116,32 bit,DMA

				flowControlParameters->MaxThroughput=0xABE0AUL;

				flowControlParameters->MaxPacketCount=0x1D1UL;

				flowControlParameters->PacketCost=0x00UL;

				flowControlParameters->BurstPeriod=0x30UL;

				flowControlParameters->IntDeas=0x0A;

				break;

			default:break;//make lint happy

			}

		}

	} else {

		if(Platform_Is16BitMode(platformData)) {

			switch(platformData->dwIdRev&0xFFFF0000) {

			case 0x01180000UL:

			case 0x01170000UL:

			case 0x01120000UL:

			case 0x92110000UL: // Boylston Lite, not tested

			case 0x92210000UL: // Bedford, not tested

				//117/118,16 bit,PIO

				flowControlParameters->MaxThroughput=(0x9F4F6UL);

				flowControlParameters->MaxPacketCount=(0x1AFUL);

				flowControlParameters->PacketCost=(0x181UL);

				flowControlParameters->BurstPeriod=(0x32UL);

				flowControlParameters->IntDeas=(0x45UL);

				break;

			case 0x01160000UL:

			case 0x01150000UL:

			case 0x92100000UL: // Boylston Lite, not tested

			case 0x92200000UL: // Bedford, not tested

				//115/116,16 bit,PIO

				flowControlParameters->MaxThroughput=(0x787FCUL);

				flowControlParameters->MaxPacketCount=(0x146UL);

				flowControlParameters->PacketCost=(0x112UL);

				flowControlParameters->BurstPeriod=(0x40UL);

				flowControlParameters->IntDeas=(0x36UL);

				break;

			default:break;//make lint happy

			}

		} else {

			switch(platformData->dwIdRev&0xFFFF0000) {

			case 0x01180000UL:

			case 0x01170000UL:

			case 0x01120000UL:

				//117/118,32 bit,PIO

				flowControlParameters->MaxThroughput=(0xAE5C8UL);

				flowControlParameters->MaxPacketCount=(0x1D8UL);

				flowControlParameters->PacketCost=(0UL);

				flowControlParameters->BurstPeriod=(0x57UL);

				flowControlParameters->IntDeas=(0x14UL);

				break;

			case 0x01160000UL:

			case 0x01150000UL:

				//115/116,32 bit,PIO

				flowControlParameters->MaxThroughput=(0x9E338UL);

				flowControlParameters->MaxPacketCount=(0x1ACUL);

				flowControlParameters->PacketCost=(0xD2UL);

				flowControlParameters->BurstPeriod=(0x60UL);

				flowControlParameters->IntDeas=(0x0EUL);

				break;

			default:break;//make lint happy

			}

		}

	}

}



BOOLEAN Platform_RequestIRQ(

	PPLATFORM_DATA platformData,

	DWORD dwIrq,

	irqreturn_t (*pIsr)(int a,void *b,struct pt_regs *c),

	void * dev_id)

{

	BOOLEAN result=FALSE;

	SMSC_ASSERT(platformData!=NULL);

	SMSC_ASSERT(platformData->dev_id==NULL);

	if(request_irq(

		dwIrq,

		pIsr,

		SA_INTERRUPT,

		"SMSC_LAN911x_ISR",

		dev_id)!=0)

	{

		SMSC_WARNING("Unable to use IRQ = %ld",dwIrq);

		goto DONE;

	}

	platformData->dwIrq=dwIrq;

	platformData->dev_id=dev_id;

	result=TRUE;

DONE:

	return result;

}



DWORD Platform_CurrentIRQ(

	PPLATFORM_DATA platformData)

{

	SMSC_ASSERT(platformData!=NULL);

	return platformData->dwIrq;

}



void Platform_FreeIRQ(

	PPLATFORM_DATA platformData)

{

	if(platformData==NULL) {

		SMSC_ASSERT(FALSE);

		return;

	}

	if(platformData->dev_id==NULL) {

		SMSC_ASSERT(FALSE);

		return;

	}

	

	free_irq(platformData->dwIrq,platformData->dev_id);



	platformData->dwIrq=0;

	platformData->dev_id=NULL;	

}



BOOLEAN Platform_IsValidDmaChannel(DWORD dwDmaCh)

{

	if(dwDmaCh<=3) {

		return TRUE;

	}

	return FALSE;

}



#define GetRegDW(dwBase,dwOffset) \

	((DWORD)(*(volatile DWORD *)(dwBase + dwOffset)))



#define SetRegDW(dwBase,dwOffset,dwVal) \

	((*(volatile DWORD *)(dwBase + dwOffset)) = dwVal)





BOOLEAN Platform_DmaDisable(

	PPLATFORM_DATA platformData,

	const DWORD dwDmaCh)

{

	const DWORD	dwDmaCh0CtrlReg = 0xA400002CUL;

	DWORD		dwTemp;



	platformData=platformData;//make lint happy



	if (dwDmaCh > 3UL)

	{

		SMSC_WARNING("Platform_DmaDisable(dwDmaCh=0x%lX), Invalid DMA Channel!\n",dwDmaCh);

		return FALSE;

	}



	dwTemp = GetRegDW(dwDmaCh0CtrlReg,(dwDmaCh * 0x10UL)); 

	dwTemp &= ~DMAC_ENBL_;	// Disable DMA controller



	SetRegDW(dwDmaCh0CtrlReg, (dwDmaCh * 0x10UL), dwTemp); 



	return TRUE;

}



void Platform_CacheInvalidate(

	PPLATFORM_DATA platformData,

	const void * const pStartAddress,

	const DWORD dwLengthInBytes)

{

	Platform_CachePurge(

		platformData,

		pStartAddress,

		dwLengthInBytes);

}



void Platform_CachePurge(

	PPLATFORM_DATA platformData,

	const void * const pStartAddress,

	const DWORD dwLengthInBytes)

{

	DWORD	dwAddrIndex;

	DWORD	begin, end;



	const DWORD	dwStartAddr = CpuToPhysicalAddr(pStartAddress);



	platformData=platformData;//make lint happy



	begin = dwStartAddr & ~(PLATFORM_CACHE_LINE_BYTES - 1UL);

	end = (dwStartAddr + dwLengthInBytes + PLATFORM_CACHE_LINE_BYTES - 1UL) & 

				~(PLATFORM_CACHE_LINE_BYTES - 1UL);



	for (dwAddrIndex = begin; 

		dwAddrIndex < end; 

		dwAddrIndex += PLATFORM_CACHE_LINE_BYTES)

	{

		DWORD	dwData, dwAddr;



		dwData = (dwAddrIndex & CACHE_PHYSADDR_MASK);	 /* _Virtual_ address, ~U, ~V */

		dwAddr = CACHE_OC_ADDRESS_ARRAY | (dwAddrIndex & CACHE_OC_ENTRY_MASK) | 

				CACHE_OC_ASSOC_BIT_;

		(*(volatile unsigned long *)dwAddr) = dwData;

	}

}



DWORD Platform_RequestDmaChannel(

	PPLATFORM_DATA platformData)

{

	platformData=platformData;//make lint happy

	return TRANSFER_PIO;

}



void Platform_ReleaseDmaChannel(

	PPLATFORM_DATA platformData,

	DWORD dwDmaChannel)

{

	platformData=platformData;//make lint happy

	dwDmaChannel=dwDmaChannel;//make lint happy

	//since Platform_RequestDmaChannel

	//  never returns a dma channel

	//  then this function should never be called

	SMSC_ASSERT(FALSE);

}



void Platform_DmaComplete(

	PPLATFORM_DATA platformData,

	const DWORD dwDmaCh)

{

	DWORD dwTimeOut=1000000;

	SMSC_ASSERT(dwDmaCh<4);

	while((Platform_DmaGetDwCnt(platformData,dwDmaCh))&&(dwTimeOut))

	{

		udelay(1);

		dwTimeOut--;

	}

	if(!Platform_DmaDisable(platformData,dwDmaCh)) {

		SMSC_WARNING("Failed Platform_DmaDisable");

	}

	if(dwTimeOut==0) {

		SMSC_WARNING("Platform_DmaComplete: Timed out");

	}

}



DWORD Platform_DmaGetDwCnt(

	PPLATFORM_DATA platformData,

	const DWORD dwDmaCh)

{

	const DWORD	dwDmaCh0XferCntReg = 0xA4000028UL;



	platformData=platformData;//make lint happy





	if (dwDmaCh > 3UL)

	{

		SMSC_WARNING("DmaGetDwCnt(dwDmaCh=0x%lX), Invalid DMA Channel!\n",dwDmaCh);

		return 0UL;

	}



	return (GetRegDW(dwDmaCh0XferCntReg,(dwDmaCh * 0x10UL)));

}



BOOLEAN Platform_DmaInitialize(

	PPLATFORM_DATA platformData,

	DWORD dwDmaCh)

{

	platformData=platformData;//make lint happy

	dwDmaCh=dwDmaCh;//make lint happy

	DMATCR0=0;

	DMATCR1=0;

	DMATCR2=0;

	DMATCR3=0;

	DMAOR|=DMA_MSTR_EN_;

	return TRUE;

}



BOOLEAN Platform_DmaStartXfer(

	PPLATFORM_DATA platformData,

	const DMA_XFER * const pDmaXfer)

{

	const DWORD	dwDmaSrcBaseReg = (0xA4000020UL + (pDmaXfer->dwDmaCh * 0x10UL));

	const DWORD	dwDmaDstBaseReg = dwDmaSrcBaseReg + 0x04UL;

	const DWORD	dwDmaXferCntReg = dwDmaSrcBaseReg + 0x08UL;

	const DWORD	dwDmaCtrlReg    = dwDmaSrcBaseReg + 0x0CUL;



	DWORD dwTemp, dwSrcAddr, dwDestAddr;

	DWORD dwAlignMask;

	DWORD dwLanPhysAddr, dwMemPhysAddr;



	platformData=platformData;//make lint happy



	// 1. validate the requested channel #

	if (pDmaXfer->dwDmaCh > 3UL)

	{

		SMSC_WARNING("Platform_DmaStartXfer -- bad dwDmaCh=%ld", pDmaXfer->dwDmaCh);

		return FALSE;

	}



	// 2. make sure the channel's not already running

	if (GetRegDW(dwDmaXferCntReg,0) != 0UL)

	{

		SMSC_WARNING("Platform_DmaStartXfer -- requested channel (%ld) is still running", pDmaXfer->dwDmaCh);

		return FALSE;

	}



	// 3. calculate the physical transfer addresses

	dwLanPhysAddr = CpuToPhysicalAddr((void *)pDmaXfer->dwLanReg);

	dwMemPhysAddr = CpuToPhysicalAddr((void *)pDmaXfer->pdwBuf);



	// 4. validate the address alignments

	// need CL alignment for CL bursts

	dwAlignMask = (PLATFORM_CACHE_LINE_BYTES - 1UL);

	

	if ((dwLanPhysAddr & dwAlignMask) != 0UL)

	{

		SMSC_WARNING("Platform_DmaStartXfer -- bad dwLanPhysAddr (0x%08lX) alignment",

			dwLanPhysAddr);

		return FALSE;

	}



	if ((dwMemPhysAddr & dwAlignMask) != 0UL)

	{

		SMSC_WARNING("Platform_DmaStartXfer -- bad dwMemPhysAddr (0x%08lX) alignment",

			dwMemPhysAddr);

		return FALSE;

	}



	// 5. validate the transfer size & alignment

	if (pDmaXfer->dwDwCnt >= (1UL<<24))

	{

		SMSC_WARNING("Platform_DmaStartXfer -- dwDwCnt =%ld is too big (16777216 counts max on SH3)",

			pDmaXfer->dwDwCnt);

		return FALSE;

	}



	if ((pDmaXfer->dwDwCnt & 3UL) != 0UL)

	{

		SMSC_WARNING("Platform_DmaStartXfer -- bad dwDwCnt=%ld alignment",

			pDmaXfer->dwDwCnt);

		return FALSE;

	}



	// 6. Config Control reg

	dwTemp = GetRegDW(dwDmaCtrlReg,0); 



	// Disable the selected channel first

	dwTemp &= ~DMAC_ENBL_;

	SetRegDW((dwDmaCtrlReg),0, dwTemp); 



	// Select correct ch and set SRC, DST and counter

	if (pDmaXfer->fMemWr == TRUE)

	{

		dwSrcAddr = dwLanPhysAddr;

		dwDestAddr = dwMemPhysAddr;



		dwTemp &= ~DST_ADDR_MODE_;

		dwTemp |= INC_DST_ADDR;		// Always to increment mem buffer



		dwTemp &= ~SRC_ADDR_MODE_;

	}

	else

	{

		dwSrcAddr = dwMemPhysAddr;

		dwDestAddr = dwLanPhysAddr;



		dwTemp &= ~SRC_ADDR_MODE_;

		dwTemp |= INC_SRC_ADDR;		// Always increment mem buffer



		dwTemp &= ~DST_ADDR_MODE_;

	}





	// Set Source and destination addresses

	SetRegDW(dwDmaSrcBaseReg,0, dwSrcAddr);

	SetRegDW(dwDmaDstBaseReg,0, dwDestAddr); 



	// Set the transmit size in terms of the xfer mode

	dwTemp &= ~XMIT_SIZE_;



	dwTemp &= ~XMIT_MODE_;	// Clear to 0:  Cycle-steal mode



	dwTemp |= XMIT_QDWORD;	// Set 16 bytes (QDWORD) xfer

	SetRegDW(dwDmaXferCntReg,0, (pDmaXfer->dwDwCnt/4UL)); 



	// bugbug (nw)  4/23/2004 -- these somehow were down below the section that sets the counts???

	dwTemp &= ~XMIT_RESRC_;

	dwTemp |= AUTO_REQ;		// Auto request

	dwTemp &= ~XFER_END_;

	SetRegDW(dwDmaCtrlReg,0, dwTemp); 

	// bugbug (nw)  4/23/2004 end of bugfix



	// Clrea NMIF and AE bits and enable DMA master

	DMAOR &= ~(NMI_FLAG_ | ADDR_ERR_FLAG_);



	// Enable DMA controller ch x

	dwTemp = GetRegDW(dwDmaCtrlReg,0); 

	dwTemp |= DMAC_ENBL_;

	SetRegDW(dwDmaCtrlReg,0, dwTemp);



	// DMA Transfering....



	return TRUE;

}



void Platform_WriteFifo(

	DWORD dwLanBase,

	DWORD *pdwBuf,

	DWORD dwDwordCount)

{

	volatile DWORD *pdwReg;

	pdwReg = (volatile DWORD *)(dwLanBase+TX_DATA_FIFO);

	while(dwDwordCount)

	{

		*pdwReg = *pdwBuf++;

		dwDwordCount--;

	}

}

void Platform_ReadFifo(

	DWORD dwLanBase,

	DWORD *pdwBuf,

	DWORD dwDwordCount)

{

	const volatile DWORD * const pdwReg = 

		(const volatile DWORD * const)(dwLanBase+RX_DATA_FIFO);

	

	while (dwDwordCount)

	{

		*pdwBuf++ = *pdwReg;

		dwDwordCount--;

	}

}





